@@ -19,12 +19,15 @@ import json
 import sys
 
 from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib.i18n import _
 from osc_lib import utils
 import yaml
 
 import ironic_inspector_client
 from ironic_inspector_client import resource as res
 
+
 API_NAME = 'baremetal_introspection'
 API_VERSION_OPTION = 'inspector_api_version'
 DEFAULT_API_VERSION = '1'
@@ -37,8 +40,14 @@ for mversion in range(ironic_inspector_client.MAX_API_VERSION[1] + 1):
 
 
 def make_client(instance):
+    url = instance.get_configuration().get('inspector_url')
+    if not url:
+        url = instance.get_endpoint_for_service_type(
+            'baremetal-introspection', interface=instance.interface,
+            region_name=instance._region_name
+        )
     return ironic_inspector_client.ClientV1(
-        inspector_url=instance.get_configuration().get('inspector_url'),
+        inspector_url=url,
         session=instance.session,
         api_version=instance._api_version[API_NAME],
         interface=instance._interface,
@@ -46,6 +55,7 @@ def make_client(instance):
 
 
 def build_option_parser(parser):
+    # TODO(dtantsur): deprecate these options in favor of more generic OS_*
     parser.add_argument('--inspector-api-version',
                         default=utils.env('INSPECTOR_VERSION',
                                           default=DEFAULT_API_VERSION),
@@ -71,9 +81,19 @@ class StartCommand(command.Lister):
                             action='store_true',
                             help='wait for introspection to finish; the result'
                             ' will be displayed in the end')
+        parser.add_argument('--check-errors',
+                            action='store_true',
+                            help='check if errors occurred during the'
+                            ' introspection; if any error occurs only the'
+                            ' errors are displayed; can only be used with'
+                            ' --wait')
         return parser
 
     def take_action(self, parsed_args):
+        if parsed_args.check_errors and not parsed_args.wait:
+            raise exceptions.CommandError(
+                _("--check-errors can only be used with --wait"))
+
         client = self.app.client_manager.baremetal_introspection
         for uuid in parsed_args.node:
             client.introspect(uuid)
@@ -83,6 +103,14 @@ class StartCommand(command.Lister):
             result = client.wait_for_finish(parsed_args.node)
             result = [(uuid, s.get('error'))
                       for uuid, s in result.items()]
+            if parsed_args.check_errors:
+                uuids_errors = "\n".join("%s (%s)" % node_info
+                                         for node_info in result
+                                         if node_info[1] is not None)
+                if uuids_errors:
+                    raise Exception(
+                        _("Introspection failed for some nodes: %s")
+                        % uuids_errors)
         else:
             result = []
 
